import trafaret as t


def xor_key(first, second, trafaret):
    trafaret = t.Trafaret._trafaret(trafaret)

    def check_(value):
        if (first in value) ^ (second in value):
            key = first if first in value else second
            yield first, t.catch_error(trafaret, value[key]), (key,)
        elif first in value and second in value:
            yield first, t.DataError(error=f'correct only if {second} is not defined'), (first,)
            yield second, t.DataError(error=f'correct only if {first} is not defined'), (second,)
        else:
            yield first, t.DataError(error=f'is required if {second} is not defined'), (first,)
            yield second, t.DataError(error=f'is required if {first} is not defined'), (second,)

    return check_


def confirm_key(name, confirm_name, trafaret):
    def check_(value):
        first, second = None, None
        if name in value:
            first = value[name]
        else:
            yield name, t.DataError('is required'), (name,)
        if confirm_name in value:
            second = value[confirm_name]
        else:
            yield confirm_name, t.DataError('is required'), (confirm_name,)
        if not (first and second):
            return
        yield name, t.catch_error(trafaret, first), (name,)
        yield confirm_name, t.catch_error(trafaret, second), (confirm_name,)
        if first != second:
            yield confirm_name, t.DataError(f'must be equal to {name}'), (confirm_name,)
    return check_



from dateutil.parser import parse
from datetime import datetime
from .. import Trafaret, str_types


class DateTime(Trafaret):
    """ Class for support parsing date im RFC3339 formats
        via dateutil.parse helper
    """
    convertable = str_types + (datetime,)
    value_type = datetime

    def __init__(self, allow_blank=False):
        self.allow_blank = allow_blank

    def __repr__(self):
        return "<Date(blank)>" if self.allow_blank else "<Date>"

    def check_and_return(self, value):
        if isinstance(value, datetime):
            return value
        try:
            return parse(value)
        except ValueError as e:
            self._failure(e.message)
